var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ChangeDetectionStrategy, Input, Output, EventEmitter, HostListener, ElementRef, TemplateRef } from '@angular/core';
import { WindowRef } from '../window-ref/window-ref.service';
var InfiniteScrollComponent = (function () {
    function InfiniteScrollComponent(windowRef, elementRef) {
        this.windowRef = windowRef;
        this.elementRef = elementRef;
        this.bufferPx = 40;
        this.scrollDelay = 300;
        this.loadMore = new EventEmitter();
        this.scrollStream = new EventEmitter();
    }
    InfiniteScrollComponent.prototype.ngOnInit = function () {
        this.setupScrollEvent();
    };
    InfiniteScrollComponent.prototype.loadFinish = function (complete) {
        var _this = this;
        if (complete) {
            return this.unSubscription();
        }
        setTimeout(function (_) { return _this.onWinScroll(null); }, this.scrollDelay);
    };
    InfiniteScrollComponent.prototype.setupScrollEvent = function () {
        var _this = this;
        if (!this.subscription) {
            this.subscription = this.registerOnScrollStream(this.scrollStream)
                .subscribe(function (_) { return _this.onScrollChange(); });
        }
    };
    InfiniteScrollComponent.prototype.onScrollChange = function () {
        var winHeight = this.windowRef.innerHeight;
        var clientRect = this.windowRef.getBoundingClientRect(this.elementRef);
        if (clientRect && winHeight - clientRect.top >= this.bufferPx) {
            this.loadMore.emit(this);
        }
    };
    InfiniteScrollComponent.prototype.onWinScroll = function ($event) {
        this.scrollStream.emit($event);
    };
    InfiniteScrollComponent.prototype.ngOnDestroy = function () {
        this.unSubscription();
    };
    InfiniteScrollComponent.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    InfiniteScrollComponent.prototype.registerOnScrollStream = function (scrollStream) {
        return scrollStream
            .debounceTime(this.scrollDelay)
            .distinctUntilChanged();
    };
    return InfiniteScrollComponent;
}());
__decorate([
    Input(),
    __metadata("design:type", Number)
], InfiniteScrollComponent.prototype, "total", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], InfiniteScrollComponent.prototype, "bufferPx", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], InfiniteScrollComponent.prototype, "scrollDelay", void 0);
__decorate([
    Input(),
    __metadata("design:type", TemplateRef)
], InfiniteScrollComponent.prototype, "spinnerTemplate", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], InfiniteScrollComponent.prototype, "loadMore", void 0);
__decorate([
    HostListener('window:scroll', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Event]),
    __metadata("design:returntype", void 0)
], InfiniteScrollComponent.prototype, "onWinScroll", null);
InfiniteScrollComponent = __decorate([
    Component({
        selector: 're-infinite-scroll',
        template: "\n    <template #defaultTemplate let-spinner=\"\">\n      <div class=\"spinner\">\n        <div class=\"bounce bounce1\"></div>\n        <div class=\"bounce bounce2\"></div>\n        <div class=\"bounce bounce3\"></div>\n      </div>\n    </template>\n\n    <template [ngTemplateOutlet]=\"spinnerTemplate || defaultTemplate\" [ngOutletContext]=\"{ $implicit: this }\"></template>\n  ",
        styles: ["\n    .spinner{margin:10px auto 0px;width:150px;text-align:center}.spinner .bounce{width:24px;height:24px;background-color:#E3C381;border-radius:100%;display:inline-block;-webkit-animation:bouncedelay 1.4s infinite ease-in-out;animation:bouncedelay 1.4s infinite ease-in-out;-webkit-animation-fill-mode:both;animation-fill-mode:both}.spinner .bounce1{-webkit-animation-delay:-0.32s;animation-delay:-0.32s}.spinner .bounce2{-webkit-animation-delay:-0.16s;animation-delay:-0.16s}@-webkit-keyframes bouncedelay{0%,80%,100%{-webkit-transform:scale(0)}40%{-webkit-transform:scale(1)}}@keyframes bouncedelay{0%,80%,100%{transform:scale(0);-webkit-transform:scale(0)}40%{transform:scale(1);-webkit-transform:scale(1)}}\n  "],
        changeDetection: ChangeDetectionStrategy.OnPush,
        exportAs: 'infiniteScroll'
    }),
    __metadata("design:paramtypes", [WindowRef, ElementRef])
], InfiniteScrollComponent);
export { InfiniteScrollComponent };
//# sourceMappingURL=infinite-scroll.component.js.map